extern crate regex;

use std::collections::LinkedList;
use crate::utils;
use regex::Regex;

pub fn execute_task01(content: &str) {
    let line = solve_task01(content);

    println!("Day05 - Task01 - Last Item Str Line: {}", line);
}

pub fn execute_task02(content: &str) {
    let line = solve_task02(content);

    println!("Day05 - Task02 - Last Item Str Line: {}", line);
}

fn solve_task01(content: &str) -> String {
    let re = Regex::new(r"move (\d+) from (\d+) to (\d+)").unwrap();

    let mut setup: Vec<&str> = vec![];
    let mut setup_finish = false;
    let mut port: Vec<LinkedList<String>> = Default::default();
    let vector = utils::convert_to_string_slice(&content);

    vector.into_iter().for_each(|line| {
        if line.trim().is_empty() {
            setup_finish = true;
            port = parse_port(setup.clone().to_vec());
            return;
        }

        if !setup_finish {
            setup.push(line);
            return;
        }

        let caps = re.captures(line).unwrap();
        let size: i32 = caps.get(1).map(|content| content.as_str().parse::<i32>()).unwrap().unwrap();
        let from: &str = caps.get(2).map(|content| content.as_str()).unwrap();
        let to: &str = caps.get(3).map(|content| content.as_str()).unwrap();

        move_in_port(&mut port, from, to, size)
    });

    let str_line_vec: Vec<String> = port.into_iter().map(|vec| vec.into_iter().last().unwrap().trim().replace("[", "").replace("]", "")).collect();
    str_line_vec.join("")
}

fn solve_task02(content: &str) -> String {
    let re = Regex::new(r"move (\d+) from (\d+) to (\d+)").unwrap();

    let mut setup: Vec<&str> = vec![];
    let mut setup_finish = false;
    let mut port: Vec<LinkedList<String>> = Default::default();
    let vector = utils::convert_to_string_slice(&content);

    vector.into_iter().for_each(|line| {
        if line.trim().is_empty() {
            setup_finish = true;
            port = parse_port(setup.clone().to_vec());
            return;
        }

        if !setup_finish {
            setup.push(line);
            return;
        }

        let caps = re.captures(line).unwrap();
        let size: i32 = caps.get(1).map(|content| content.as_str().parse::<i32>()).unwrap().unwrap();
        let from: &str = caps.get(2).map(|content| content.as_str()).unwrap();
        let to: &str = caps.get(3).map(|content| content.as_str()).unwrap();

        move_in_port_currently(&mut port, from, to, size)
    });

    let str_line_vec: Vec<String> = port.into_iter().map(|vec| vec.into_iter().last().unwrap().trim().replace("[", "").replace("]", "")).collect();

    str_line_vec.join("")
}


fn move_in_port(mut port: &mut Vec<LinkedList<String>>, from: &str, to: &str, size: i32) {
    for _ in 0..size {
        move_in_port_once(&mut port, from, to)
    }
}

fn move_in_port_currently(port: &mut Vec<LinkedList<String>>, from: &str, to: &str, size: i32) {
    let mut tmp: LinkedList<String> = LinkedList::new();
    for _ in 0..size {
        let tmp_tmp = port.get_mut((from.parse::<usize>().unwrap()) -1).unwrap().pop_back().unwrap();
        tmp.push_front(tmp_tmp);
    }

    port.get_mut((to.parse::<usize>().unwrap()) -1).unwrap().append(&mut tmp);
}

fn move_in_port_once(port: &mut Vec<LinkedList<String>>, from: &str, to: &str) {
    let tmp: String = port.get_mut((from.parse::<usize>().unwrap()) -1).unwrap().pop_back().unwrap();
    port.get_mut((to.parse::<usize>().unwrap()) -1).unwrap().push_back(tmp);
}

fn parse_port(lines: Vec<&str>) -> Vec<LinkedList<String>> {
    let mut port = vec![];

    lines.into_iter()
        .rev()
        .skip(1)
        .map(|line| parse_port_level(line))
        .for_each(|containers| {
            for (i, item) in containers.into_iter().enumerate() {
                if item.trim().is_empty() {
                    continue;
                }

                if port.len() <= i {
                    port.insert(i, LinkedList::new());
                }

                port.get_mut(i).unwrap().push_back(item);
            }
        });

    return port;
}

fn parse_port_level(line: &str) -> Vec<String> {
    line.chars()
        .collect::<Vec<char>>()
        .chunks(4)
        .map(|c| c.iter().collect::<String>())
        .collect()
}


#[cfg(test)]
mod test {

    const EXAMPLE01: &'static str = include_str!("input/day05/example01");


    #[test]
    fn test_first() {
        assert_eq!(super::solve_task01(EXAMPLE01), "CMZ");
    }

#[test]
fn test_second() {
        assert_eq!(super::solve_task02(EXAMPLE01), "MCD");
    }
}