use std::collections::HashSet;

pub fn execute_task01(content: &str) {
    let index = solve_task01(content);

    println!("Day06 - Task01 - First code start at: {}", index);
}

pub fn execute_task02(content: &str) {
    let index = solve_task02(content);

    println!("Day06 - Task02 - First code start at: {}", index);
}

fn solve_task01(content: &str) -> usize {
    find_code(content, 4)
}

fn solve_task02(content: &str) -> usize {
    find_code(content, 14)
}

fn find_code(content: &str, len: usize) -> usize {
    content
    .as_bytes()
    .windows(len)
    .enumerate()
    .filter(|(_, part)| {
        let mut used = HashSet::new();
        part.iter().all(move |elt| used.insert(elt))
    })
    .map(|(i, _)| i + len)
    .next().unwrap()
}


#[cfg(test)]
mod test {

    #[test]
    fn test_first() {
        assert_eq!(super::solve_task01("bvwbjplbgvbhsrlpgdmjqwftvncz"), 5);
        assert_eq!(super::solve_task01("nppdvjthqldpwncqszvftbrmjlhg"), 6);
        assert_eq!(super::solve_task01("nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg"), 10);
        assert_eq!(super::solve_task01("zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw"), 11);
    }

    #[test]
    fn test_second() {
        assert_eq!(super::solve_task02("mjqjpqmgbljsphdztnvjfqwrcgsmlb"), 19);
        assert_eq!(super::solve_task02("bvwbjplbgvbhsrlpgdmjqwftvncz"), 23);
        assert_eq!(super::solve_task02("nppdvjthqldpwncqszvftbrmjlhg"), 23);
        assert_eq!(super::solve_task02("nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg"), 29);
        assert_eq!(super::solve_task02("zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw"), 26);
    }
}